'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _includes2 = require('lodash/includes');

var _includes3 = _interopRequireDefault(_includes2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _graphql = require('graphql');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _queryRunner = require('../utils/query-runner');

var _queryRunner2 = _interopRequireDefault(_queryRunner);

var _globals = require('../utils/globals');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _createPath = require('./create-path');

var _createPath2 = _interopRequireDefault(_createPath);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _joi = require('joi');

var _joi2 = _interopRequireDefault(_joi);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _joi3 = require('../joi-schemas/joi');

var _apiRunnerNode = require('../utils/api-runner-node');

var _apiRunnerNode2 = _interopRequireDefault(_apiRunnerNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('../utils/js-chunk-names'),
    layoutComponentChunkName = _require.layoutComponentChunkName; /*  weak */


var mkdirs = _bluebird2.default.promisify(_fsExtra2.default.mkdirs);
var copy = _bluebird2.default.promisify(_fsExtra2.default.copy);
var removeDir = _bluebird2.default.promisify(_fsExtra2.default.remove);

// Joi schemas


_bluebird2.default.onPossiblyUnhandledRejection(function (error) {
  throw error;
});
process.on('unhandledRejection', function (error) {
  console.error('UNHANDLED REJECTION', error.stack);
});

// Path creator.
// Auto-create pages.
// algorithm is glob /pages directory for js/jsx/cjsx files *not*
// underscored. Then create url w/ our path algorithm *unless* user
// takes control of that page component in gatsby-node.
var autoPathCreator = function autoPathCreator(program) {
  return new _bluebird2.default(function (resolve) {
    var pagesDirectory = _path2.default.join(program.directory, 'pages');
    var autoPages = [];
    (0, _glob2.default)(pagesDirectory + '/**/?(*.js|*.jsx|*.cjsx)', function (err, files) {
      // Create initial page objects.
      autoPages = files.map(function (filePath) {
        return {
          component: filePath,
          componentChunkName: layoutComponentChunkName(program.directory, filePath),
          path: filePath
        };
      });

      // Convert path to one relative to the pages directory.
      autoPages = autoPages.map(function (page) {
        return (0, _extends3.default)({}, page, {
          path: _path2.default.relative(pagesDirectory, page.path)
        });
      });

      // Remove pages starting with an underscore.
      autoPages = (0, _filter3.default)(autoPages, function (page) {
        return page.path.slice(0, 1) !== '_';
      });

      // Remove page templates.
      autoPages = (0, _filter3.default)(autoPages, function (page) {
        return !(0, _includes3.default)(page.path, 'PAGE_TEMPLATE');
      });

      // Convert to our path format.
      autoPages = autoPages.map(function (page) {
        return (0, _extends3.default)({}, page, {
          path: (0, _createPath2.default)(pagesDirectory, page.component)
        });
      });

      // Validate pages.
      autoPages.forEach(function (page) {
        var _Joi$validate = _joi2.default.validate(page, _joi3.pageSchema),
            error = _Joi$validate.error;

        if (error) {
          console.log(_chalk2.default.blue.bgYellow('A page object failed validation'));
          console.log(page);
          console.log(_chalk2.default.bold.red(error));
        }
      });

      resolve(autoPages);
    });
  });
};

module.exports = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(program, cb) {
    var config, normalizedConfig, result, srcDir, siteDir, schema, graphqlRunner, pages, autoPages, modifiedPages;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // Set the program to the globals programDB
            (0, _globals.programDB)(program);

            // Try opening the site's gatsby-config.js file.
            config = {};

            try {
              config = require(program.directory + '/gatsby-config');
            } catch (e) {
              console.log('Couldn\'t open your gatsby-config.js file');
              console.log(e);
              process.exit();
            }

            // Add config to site object.
            normalizedConfig = void 0;

            if (config.default) {
              normalizedConfig = config.default;
              (0, _globals.siteDB)((0, _globals.siteDB)().set('config', config.default));
            } else {
              normalizedConfig = config;
              (0, _globals.siteDB)((0, _globals.siteDB)().set('config', config));
            }

            // Validate gatsby-config.js
            result = _joi2.default.validate(normalizedConfig, _joi3.gatsbyConfigSchema);

            if (result.error) {
              console.log(_chalk2.default.blue.bgYellow('gatsby.config.js failed validation'));
              console.log(_chalk2.default.bold.red(result.error));
              console.log(normalizedConfig);
              process.exit();
            }

            // Copy our site files to the root of the site.
            srcDir = __dirname + '/../intermediate-representation-dir';
            siteDir = program.directory + '/.intermediate-representation';
            _context.prev = 9;
            _context.next = 12;
            return removeDir(siteDir);

          case 12:
            _context.next = 14;
            return copy(srcDir, siteDir, { clobber: true });

          case 14:
            _context.next = 16;
            return mkdirs(program.directory + '/.intermediate-representation/json');

          case 16:
            _context.next = 22;
            break;

          case 18:
            _context.prev = 18;
            _context.t0 = _context['catch'](9);

            console.log('Unable to copy site files to .intermediate-representation');
            console.log(_context.t0);

          case 22:
            _context.next = 24;
            return require('../schema')();

          case 24:
            schema = _context.sent;

            graphqlRunner = function graphqlRunner(query, context) {
              return (0, _graphql.graphql)(schema, query, context, context, context);
            };

            // TODO create new folder structure for files to reflect major systems.
            // TODO validate page objects when adding them.
            // TODO validate gatsby-config.js.
            // TODO pass names of layout/page components into commons plugin for proper
            // code splitting.
            // TODO split layout components into their own files as well.
            // TODO add plugin system plus ways to use Redux + Glamor server rendering.
            // TODO layouts can specify parent layouts
            // TODO js pages + markdown can specify layouts directly.
            //
            // directory structure
            // /pages --> source plugins by default turn these files into pages.
            // /drafts --> source plugins can have a development option which would add
            //   drafts but only in dev server.
            // /layouts --> layout components e.g. blog-post.js, tag-page.js
            // /components --> this is convention only but dumping ground for react.js components
            // /data --> default json, yaml, csv, toml source plugins extract data from these.
            // /assets --> anything here is copied verbatim to /public
            // /public --> build place
            // / --> html.js, gatsby-node.js, gatsby-browser.js, gatsby-config.js

            // Collect pages.


            _context.next = 28;
            return (0, _apiRunnerNode2.default)('createPages', { graphql: graphqlRunner }, []);

          case 28:
            pages = _context.sent;


            // Add chunkName.
            pages.forEach(function (page) {
              return page.componentChunkName = layoutComponentChunkName(program.directory, page.component);
            });

            // Validate pages.
            if (pages) {
              pages.forEach(function (page) {
                var _Joi$validate2 = _joi2.default.validate(page, _joi3.pageSchema),
                    error = _Joi$validate2.error;

                if (error) {
                  console.log(_chalk2.default.blue.bgYellow('A page object failed validation'));
                  console.log(_chalk2.default.bold.red(error));
                  console.log('page object');
                  console.log(page);
                }
              });
            }

            // Save pages to in-memory database.
            if (pages) {
              (function () {
                var pagesMap = new _map2.default();
                pages.forEach(function (page) {
                  pagesMap.set(page.path, page);
                });
                (0, _globals.pagesDB)(pagesMap);
              })();
            }

            // TODO move this to own source plugin per component type (js/cjsx/typescript, etc.)
            _context.next = 34;
            return autoPathCreator(program, pages);

          case 34:
            autoPages = _context.sent;

            if (autoPages) {
              (function () {
                var pagesMap = new _map2.default();
                autoPages.forEach(function (page) {
                  return pagesMap.set(page.path, page);
                });
                (0, _globals.pagesDB)(new _map2.default([].concat((0, _toConsumableArray3.default)((0, _globals.pagesDB)()), (0, _toConsumableArray3.default)(pagesMap))));
              })();
            }

            _context.next = 38;
            return (0, _apiRunnerNode2.default)('onPostCreatePages', (0, _globals.pagesDB)(), (0, _globals.pagesDB)());

          case 38:
            modifiedPages = _context.sent;


            // Validate pages.
            modifiedPages.forEach(function (page) {
              var _Joi$validate3 = _joi2.default.validate(page, _joi3.pageSchema),
                  error = _Joi$validate3.error;

              if (error) {
                console.log(_chalk2.default.blue.bgYellow('A page object failed validation'));
                console.log(_chalk2.default.bold.red(error));
                console.log('page object');
                console.log(page);
              }
            });

            (0, _queryRunner2.default)(program, graphqlRunner, function () {
              cb(null, schema);
            });

          case 41:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined, [[9, 18]]);
  }));

  return function (_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();