'use strict';

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _kebabCase2 = require('lodash/kebabCase');

var _kebabCase3 = _interopRequireDefault(_kebabCase2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _nodeExiftool = require('node-exiftool');

var _nodeExiftool2 = _interopRequireDefault(_nodeExiftool);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _graphql = require('graphql');

var _graphqlRelay = require('graphql-relay');

var _sharp = require('sharp');

var _sharp2 = _interopRequireDefault(_sharp);

var _md5File = require('md5-file');

var _md5File2 = _interopRequireDefault(_md5File);

var _markdownIt = require('markdown-it');

var _markdownIt2 = _interopRequireDefault(_markdownIt);

var _parseFilepath = require('parse-filepath');

var _parseFilepath2 = _interopRequireDefault(_parseFilepath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var md = new _markdownIt2.default({
  html: true,
  typographer: true
});

//import inferGraphQLType from './infer-graphql-type'

var ep = new _nodeExiftool2.default.ExiftoolProcess('/usr/local/bin/exiftool');
var isOpen = ep.open();

module.exports = function (directory) {
  return new _bluebird2.default(function (resolve, reject) {
    isOpen.catch(function (err) {
      return reject(err);
    }).then(function () {
      (0, _glob2.default)(directory + '/**/?(*.png|*.jpg|*.jpeg)', function (err, files) {
        _bluebird2.default.all(files.map(function (file) {
          return ep.readMetadata(file);
        })).then(function (results) {
          var cleanedResults = (0, _filter3.default)(results, function (result) {
            return result.error === null;
          });
          var mappedResults = cleanedResults.map(function (result) {
            return result.data[0];
          });
          mappedResults = mappedResults.map(function (image) {
            // Set path from FileName
            var parsedPath = (0, _parseFilepath2.default)(image.FileName);
            image.path = '/' + (0, _kebabCase3.default)(parsedPath.dir + '/' + parsedPath.name) + '/';
            // Render description as markdown (if set).
            if (image.Description) {
              image.Description = md.render(image.Description.replace(/<br \/>/g, '\n'));
            }
            return image;
          });
          var ImageType = new _graphql.GraphQLObjectType({
            name: 'Image',
            fields: {
              SourceFile: { type: _graphql.GraphQLString },
              FileName: { type: _graphql.GraphQLString },
              FileSize: { type: _graphql.GraphQLString },
              FileType: { type: _graphql.GraphQLString },
              ImageWidth: { type: _graphql.GraphQLInt },
              ImageHeight: { type: _graphql.GraphQLInt },
              Caption: { type: _graphql.GraphQLString },
              Description: { type: _graphql.GraphQLString },
              path: { type: _graphql.GraphQLString },
              Date: {
                type: _graphql.GraphQLString,
                args: {
                  formatString: {
                    type: _graphql.GraphQLString
                  }
                },
                resolve: function resolve(_ref, _ref2) {
                  var Date = _ref.Date;
                  var formatString = _ref2.formatString;

                  if (formatString) {
                    return (0, _moment2.default)(Date, 'YYYY:MM:DD HH:mm:ssZ').format(formatString);
                  } else {
                    return Date;
                  }
                }
              },
              image: {
                type: new _graphql.GraphQLObjectType({
                  name: 'ImageSrc',
                  fields: {
                    src: { type: _graphql.GraphQLString },
                    width: { type: _graphql.GraphQLInt },
                    height: { type: _graphql.GraphQLInt }
                  }
                }),
                args: {
                  width: {
                    type: _graphql.GraphQLInt,
                    defaultValue: 400
                  },
                  height: {
                    type: _graphql.GraphQLInt
                  },
                  quality: {
                    type: _graphql.GraphQLInt,
                    defaultValue: 50
                  },
                  grayscale: {
                    type: _graphql.GraphQLBoolean,
                    defaultValue: false
                  },
                  base64: {
                    type: _graphql.GraphQLBoolean,
                    defaultValue: false
                  }
                },
                resolve: function resolve(file, args) {
                  return new _bluebird2.default(function (resolve) {
                    (0, _md5File2.default)(file.SourceFile, function (err, hash) {
                      var imgSrc = '/images/' + hash + '-' + _querystring2.default.stringify(args) + '.' + file.FileTypeExtension;
                      var filePath = process.cwd() + '/public' + imgSrc;
                      var transformer = (0, _sharp2.default)(file.SourceFile).resize(args.width, args.height).quality(args.quality).progressive();

                      // grayscale
                      if (args.grayscale) {
                        transformer = transformer.grayscale();
                      }

                      // rotate
                      if (args.rotate) {
                        transformer = transformer.rotate(args.rotate);
                      } else {
                        // Rotate according to the EXIF Orientation tag.
                        transformer = transformer.rotate();
                      }

                      if (args.base64) {
                        transformer.toBuffer(function (err, data) {
                          resolve({
                            src: data.toString('base64'),
                            width: args.width
                          });
                        });
                      } else {
                        transformer.toFile(filePath).then(function (metadata) {
                          metadata.src = imgSrc;
                          resolve(metadata);
                        });
                      }
                    });
                  });
                }
              }
            }
          });

          var _connectionDefinition = (0, _graphqlRelay.connectionDefinitions)({
            nodeType: ImageType,
            connectionFields: function connectionFields() {
              return {
                totalCount: {
                  type: _graphql.GraphQLInt
                }
              };
            }
          }),
              imageConnection = _connectionDefinition.connectionType;

          return resolve({
            image: {
              type: ImageType,
              args: {
                path: {
                  type: new _graphql.GraphQLNonNull(_graphql.GraphQLString),
                  description: 'URL path to image'
                }
              },
              resolve: function resolve(root, args) {
                return (0, _find3.default)(mappedResults, function (page) {
                  return page.path === args.path;
                });
              }
            },
            allImages: {
              type: imageConnection,
              description: 'Images',
              args: (0, _extends3.default)({}, _graphqlRelay.connectionArgs),
              resolve: function resolve(something, args) {
                var result = (0, _graphqlRelay.connectionFromArray)(mappedResults, args);
                result.totalCount = mappedResults.length;
                return result;
              }
            }
          });
        });
      });
    });
  });
};