'use strict';

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _each2 = require('lodash/each');

var _each3 = _interopRequireDefault(_each2);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _graphql = require('graphql');

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _pagesSchema = require('./pages-schema');

var _pagesSchema2 = _interopRequireDefault(_pagesSchema);

var _inferGraphqlType = require('./infer-graphql-type');

var _inferGraphqlType2 = _interopRequireDefault(_inferGraphqlType);

var _globals = require('../utils/globals');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

module.exports = function () {
  return new _bluebird2.default(function (resolve) {
    var config = (0, _globals.siteDB)().get('config');
    // Create site/page types.
    var metadataFields = function metadataFields() {
      var fields = { empty: { type: _graphql.GraphQLBoolean } };
      // TODO make this work with sub-objects.
      if (config.siteMetadata) {
        (0, _each3.default)(config.siteMetadata, function (v, k) {
          var type = (0, _inferGraphqlType2.default)(k, v);
          if (type === 'DATE') {
            fields[k] = {
              type: _graphql.GraphQLString,
              args: {
                formatString: {
                  type: _graphql.GraphQLString
                }
              },
              resolve: function resolve(date, _ref) {
                var formatString = _ref.formatString;

                if (formatString) {
                  return (0, _moment2.default)(date).format(formatString);
                } else {
                  return date;
                }
              }
            };
          } else {
            fields[k] = { type: (0, _inferGraphqlType2.default)(k, v) };
          }
        });
      }

      if ((0, _keys2.default)(fields).length > 2) {
        delete fields.empty;
      }

      return fields;
    };

    (0, _pagesSchema2.default)().then(function (pagesTypes) {
      var siteType = new _graphql.GraphQLObjectType({
        name: 'Site',
        fields: (0, _extends3.default)({
          siteMetadata: {
            type: new _graphql.GraphQLObjectType({
              name: 'SiteMetadata',
              fields: metadataFields()
            })
          },
          development: {
            type: new _graphql.GraphQLObjectType({
              name: 'DevelopmentConfig',
              fields: {
                port: { type: _graphql.GraphQLString }
              }
            })
          },
          linkPrefix: {
            type: _graphql.GraphQLString,
            name: 'prefixLink',
            description: 'Optionally prefix site links with this'
          },
          buildTime: {
            type: _graphql.GraphQLString,
            name: 'buildTime',
            description: 'The time at which you ran the gatsby command).',
            resolve: function resolve() {
              return new Date().toJSON();
            }
          }
        }, pagesTypes)
      });

      return resolve({
        site: {
          type: siteType,
          resolve: function resolve() {
            return (0, _globals.siteDB)().get('config');
          }
        }
      });
    });
  });
};