'use strict';

var _detectPort = require('detect-port');

var _detectPort2 = _interopRequireDefault(_detectPort);

var _hapi = require('hapi');

var _hapi2 = _interopRequireDefault(_hapi);

var _boom = require('boom');

var _boom2 = _interopRequireDefault(_boom);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _server2 = _interopRequireDefault(_server);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _negotiator = require('negotiator');

var _negotiator2 = _interopRequireDefault(_negotiator);

var _parseFilepath = require('parse-filepath');

var _parseFilepath2 = _interopRequireDefault(_parseFilepath);

var _webpackRequire = require('webpack-require');

var _webpackRequire2 = _interopRequireDefault(_webpackRequire);

var _opn = require('opn');

var _opn2 = _interopRequireDefault(_opn);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _readline = require('readline');

var _readline2 = _interopRequireDefault(_readline);

var _hapiGraphql = require('hapi-graphql');

var _hapiGraphql2 = _interopRequireDefault(_hapiGraphql);

var _webpackDevServer = require('webpack-dev-server');

var _webpackDevServer2 = _interopRequireDefault(_webpackDevServer);

var _globals = require('./globals');

var _bootstrap = require('../bootstrap');

var _bootstrap2 = _interopRequireDefault(_bootstrap);

var _webpack3 = require('./webpack.config');

var _webpack4 = _interopRequireDefault(_webpack3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*  weak */
require('node-cjsx').transform();


var rlInterface = _readline2.default.createInterface({
  input: process.stdin,
  output: process.stdout
});

var debug = require('debug')('gatsby:application');

function startServer(program) {
  var directory = program.directory;

  // Load pages for the site.
  return (0, _bootstrap2.default)(program, function (err, schema) {
    // Generate random port for webpack to listen on.
    // Perhaps should check if port is open.
    var webpackPort = Math.round(Math.random() * 1000 + 1000);
    var compilerConfig = (0, _webpack4.default)(program, directory, 'develop', webpackPort);

    var compiler = (0, _webpack2.default)(compilerConfig.resolve());

    var HTMLPath = directory + '/html';
    // Check if we can't find an html component in root of site.
    if (_glob2.default.sync(HTMLPath + '.*').length === 0) {
      HTMLPath = '../isomorphic/html';
    }

    // We use the program port not the webpack-dev-server port as if you import
    // files in your html.js they won't be available through the webpack-dev-server.
    // By using the program port, requesting these imported files might accidentally work
    // as the imported files will be available in /public. TODO test how expensive
    // it'd be to do an actual static compile of the html.js on startup to avoid
    // this discprenecy between dev and prod.
    var htmlCompilerConfig = (0, _webpack4.default)(program, directory, 'develop-html', program.port);

    (0, _webpackRequire2.default)(htmlCompilerConfig.resolve(), require.resolve(HTMLPath), function (error, factory) {
      if (error) {
        console.log('Failed to require ' + directory + '/html.js');
        error.forEach(function (e) {
          console.log(e);
        });
        process.exit();
      }
      var HTML = factory();
      debug('Configuring develop server');

      var webpackDevServer = new _webpackDevServer2.default(compiler, {
        hot: true,
        quiet: false,
        noInfo: true,
        host: program.host,
        headers: {
          'Access-Control-Allow-Origin': '*'
        },
        stats: {
          colors: true
        }
      });

      // Start webpack-dev-server
      webpackDevServer.listen(webpackPort, program.host);

      var server = new _hapi2.default.Server();

      server.connection({
        host: program.host,
        port: program.port
      });

      // As our two processes (Webpack-Dev-Server + this Hapi.js server) are
      // running on different ports, we proxy requests for the bundle.js to
      // Webpack.
      server.route({
        method: 'GET',
        path: '/commons.js',
        handler: {
          proxy: {
            uri: 'http://0.0.0.0:' + webpackPort + '/commons.js',
            passThrough: true,
            xforward: true
          }
        }
      });

      server.route({
        method: 'GET',
        path: '/html/{path*}',
        handler: function handler(request, reply) {
          if (request.path === 'favicon.ico') {
            return reply(_boom2.default.notFound());
          }

          try {
            var htmlElement = _react2.default.createElement(HTML, {
              body: '',
              scripts: ['commons'],
              headComponents: []
            });
            var html = _server2.default.renderToStaticMarkup(htmlElement);
            html = '<!DOCTYPE html>\n' + html;
            return reply(html);
          } catch (e) {
            console.log(e.stack);
            throw e;
          }
        }
      });

      server.route({
        method: 'GET',
        path: '/{path*}',
        handler: {
          directory: {
            path: program.directory + '/public',
            listing: false,
            index: false
          }
        }
      });

      server.ext('onRequest', function (request, reply) {
        if (request.path === '/graphql') {
          return reply.continue();
        }

        var negotiator = new _negotiator2.default(request.raw.req);

        // Try to map the url path to match an actual path of a file on disk.
        //const parsed = parsePath(request.path)
        //const page = pagesDB().get(parsed.dirname)

        //let absolutePath = `${program.directory}/pages`
        //let path
        //if (page) {
        //path = `/${parsePath(page.component).dirname}/${parsed.basename}`
        //absolutePath += `/${parsePath(page.component).dirname}/${parsed.basename}`
        //} else {
        //path = request.path
        //absolutePath += request.path
        //}
        //let isFile = false
        //try {
        //isFile = fs.lstatSync(absolutePath).isFile()
        //} catch (e) {
        //// Ignore.
        //}

        // If the path matches a file, return that.
        //if (isFile) {
        //request.setUrl(path)
        //reply.continue()
        // Let people load the bundle.js directly.
        //} else if (request.path === `/bundle.js`) {
        if (request.path === '/bundle.js') {
          reply.continue();
        } else if (negotiator.mediaType() === 'text/html') {
          request.setUrl('/html' + request.path);
          reply.continue();
        } else {
          reply.continue();
        }
      });

      return server.register([
      // Add GraphQL support
      {
        register: _hapiGraphql2.default,
        options: {
          query: {
            graphiql: true,
            pretty: true,
            schema: schema
          },
          route: {
            path: '/graphql'
          }
        }
      }], function (er) {
        if (er) {
          console.log(er);
          process.exit();
        }

        server.start(function (e) {
          if (e) {
            if (e.code === 'EADDRINUSE') {
              // eslint-disable-next-line max-len
              console.log('Unable to start Gatsby on port ' + program.port + ' as there\'s already a process listing on that port.');
            } else {
              console.log(e);
            }

            process.exit();
          } else {
            if (program.open) {
              (0, _opn2.default)(server.info.uri);
            }
            console.log('Listening at:', server.info.uri);
          }
        });
      });
    });
  });
}

module.exports = function (program) {
  var port = typeof program.port === 'string' ? parseInt(program.port, 10) : program.port;

  (0, _detectPort2.default)(port, function (err, _port) {
    if (err) {
      console.error(err);
      process.exit();
    }

    if (port !== _port) {
      // eslint-disable-next-line max-len
      var question = 'Something is already running at port ' + port + ' \nWould you like to run the app at another port instead? [Y/n] ';

      return rlInterface.question(question, function (answer) {
        if (answer.length === 0 || answer.match(/^yes|y$/i)) {
          program.port = _port; // eslint-disable-line no-param-reassign
        }

        return startServer(program);
      });
    }

    return startServer(program);
  });
};