'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _last2 = require('lodash/last');

var _last3 = _interopRequireDefault(_last2);

var _noop2 = require('lodash/noop');

var _noop3 = _interopRequireDefault(_noop2);

var _uniq2 = require('lodash/uniq');

var _uniq3 = _interopRequireDefault(_uniq2);

var _kebabCase2 = require('lodash/kebabCase');

var _kebabCase3 = _interopRequireDefault(_kebabCase2);

var _debounce2 = require('lodash/debounce');

var _debounce3 = _interopRequireDefault(_debounce2);

var _sortBy2 = require('lodash/sortBy');

var _sortBy3 = _interopRequireDefault(_sortBy2);

var _filter2 = require('lodash/filter');

var _filter3 = _interopRequireDefault(_filter2);

var _first2 = require('lodash/first');

var _first3 = _interopRequireDefault(_first2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _isString2 = require('lodash/isString');

var _isString3 = _interopRequireDefault(_isString2);

var _groupBy2 = require('lodash/groupBy');

var _groupBy3 = _interopRequireDefault(_groupBy2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _upperFirst2 = require('lodash/upperFirst');

var _upperFirst3 = _interopRequireDefault(_upperFirst2);

var _camelCase2 = require('lodash/camelCase');

var _camelCase3 = _interopRequireDefault(_camelCase2);

var _flow2 = require('lodash/flow');

var _flow3 = _interopRequireDefault(_flow2);

var _queue = require('async/queue');

var _queue2 = _interopRequireDefault(_queue);

var _chokidar = require('chokidar');

var _chokidar2 = _interopRequireDefault(_chokidar);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _parseFilepath = require('parse-filepath');

var _parseFilepath2 = _interopRequireDefault(_parseFilepath);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _globals = require('./globals');

var _jsChunkNames = require('./js-chunk-names');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Babylon has to use a require... why?
var babylon = require('babylon');

var pascalCase = (0, _flow3.default)(_camelCase3.default, _upperFirst3.default);

var hashStr = function hashStr(str) {
  var hash = 5381,
      i = str.length;

  while (i) {
    hash = hash * 33 ^ str.charCodeAt(--i);
  } /* JavaScript does bitwise operations (like XOR, above) on 32-bit signed
     * integers. Since we want the results to be always positive, convert the
     * signed int to an unsigned by doing an unsigned bitshift. */
  return hash >>> 0;
};

// Write out routes file.
// Loop through all paths and write them out to child-routes.js
var writeChildRoutes = function writeChildRoutes() {
  var directory = (0, _globals.programDB)().directory;
  var childRoutes = '';
  var splitChildRoutes = '';

  var genChildRoute = function genChildRoute(page, noPath) {
    var pathStr = '';
    if (!noPath) {
      pathStr = 'path:\'' + page.path + '\',';
    }
    return '\n      {\n        ' + pathStr + '\n        component: ' + page.internalComponentName + ',\n      },\n    ';
  };

  var genSplitChildRoute = function genSplitChildRoute(page) {
    var noPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    var pathName = (0, _jsChunkNames.pathChunkName)(page.path);
    var layoutName = (0, _jsChunkNames.layoutComponentChunkName)((0, _globals.programDB)().directory, page.component);
    var pathStr = '';
    if (!noPath) {
      if ((0, _globals.programDB)().prefixLinks) {
        pathStr = 'path:\'' + (0, _get3.default)((0, _globals.siteDB)().get('config'), 'linkPrefix', '') + page.path + '\',';
      } else {
        pathStr = 'path:\'' + page.path + '\',';
      }
    }

    return '\n      {\n        ' + pathStr + '\n        getComponent (nextState, cb) {\n          require.ensure([], (require) => {\n            let Component = require(\'' + page.component + '\')\n            if (Component.default) {\n              Component = Component.default\n            }\n            require.ensure([], (require) => {\n              const data = require(\'./json/' + page.jsonName + '\')\n              cb(null, () => <Component {...nextState} {...data} />)\n            }, \'' + pathName + '\')\n          }, \'' + layoutName + '\')\n        }\n      },\n    ';
  };

  // Group pages under their layout component (if any).
  var defaultLayoutExists = true;
  if (_glob2.default.sync((0, _globals.programDB)().directory + '/layouts/default.*').length === 0) {
    defaultLayoutExists = false;
  }
  var groupedPages = (0, _groupBy3.default)([].concat((0, _toConsumableArray3.default)((0, _globals.pagesDB)().values())), function (page) {
    // If is a string we'll assume it's a working layout component.
    if ((0, _isString3.default)(page.layout)) {
      return page.layout;
      // If the user explicitely turns off layout, put page in undefined bucket.
    } else if (page.layout === false) {
      return 'undefined';
      // Otherwise assume the default layout should handle this page.
    } else if (defaultLayoutExists) {
      return 'default';
      // We got nothing left, undefined.
    } else {
      return 'undefined';
    }
  });
  var rootRoute = '{ childRoutes: ['; // We close this out at the bottom.
  var splitRootRoute = '{ childRoutes: ['; // We close this out at the bottom.
  (0, _forEach3.default)(groupedPages, function (pages, layout) {
    var route = '{';
    var splitRoute = '{';
    if (layout === 'undefined') {
      // If a layout isn't defined then don't provide an index route, etc.
      route += 'childRoutes: [';
      splitRoute += 'childRoutes: [';
      pages.forEach(function (page) {
        route += genChildRoute(page);
        splitRoute += genSplitChildRoute(page);
      });
      route += ']},';
      splitRoute += ']},';
      rootRoute += route;
      splitRootRoute += splitRoute;
    } else {
      (function () {
        var indexPage = void 0;
        indexPage = (0, _first3.default)((0, _filter3.default)(pages, function (page) {
          return (0, _parseFilepath2.default)(page.component).name === 'index';
        }));
        // If there's not an index page, just pick the one with the shortest path.
        // Probably a bad heuristic.
        if (!indexPage) {
          indexPage = (0, _first3.default)((0, _sortBy3.default)(pages, function (page) {
            return page.path.length;
          }));
        }
        var otherPages = (0, _filter3.default)(pages, function (page) {
          return page.path !== indexPage.path;
        });
        var route = '\n      {\n        path: \'' + indexPage.path + '\',\n        component: require(\'' + (0, _globals.programDB)().directory + '/layouts/' + layout + '\'),\n        indexRoute: ' + genChildRoute(indexPage, true) + '\n        childRoutes: [\n      ';
        var pathStr = void 0;
        if ((0, _globals.programDB)().prefixLinks) {
          pathStr = 'path:\'' + (0, _get3.default)((0, _globals.siteDB)().get('config'), 'linkPrefix', '') + indexPage.path + '\',';
        } else {
          pathStr = 'path:\'' + indexPage.path + '\',';
        }
        var splitRoute = '\n      {\n        ' + pathStr + '\n        component: require(\'' + (0, _globals.programDB)().directory + '/layouts/' + layout + '\'),\n        indexRoute: ' + genSplitChildRoute(indexPage, true) + '\n        childRoutes: [\n      ';
        pages.forEach(function (page) {
          route += genChildRoute(page);
          splitRoute += genSplitChildRoute(page);
        });
        route += ']},';
        splitRoute += ']},';
        rootRoute += route;
        splitRootRoute += splitRoute;
      })();
    }
  });
  // Close out object.
  rootRoute += ']}';
  splitRootRoute += ']}';
  var componentsStr = [].concat((0, _toConsumableArray3.default)((0, _globals.pagesDB)().values())).map(function (page) {
    return 'class ' + page.internalComponentName + ' extends React.Component {\n          render () {\n            let Component = require(\'' + page.component + '\')\n            if (Component.default) {\n              Component = Component.default\n            }\n            const data = require(\'./json/' + page.jsonName + '\')\n            return <Component {...this.props} {...data} />\n          }\n        }';
  }).join("\n");

  childRoutes = '\n    import React from \'react\'\n\n    /**\n     * Warning from React Router, caused by react-hot-loader.\n     * The warning can be safely ignored, so filter it from the console.\n     * Otherwise you\'ll see it every time something changes.\n     * See https://github.com/gaearon/react-hot-loader/issues/298\n     */\n    if (module.hot) {\n        const isString = require(\'lodash/isString\')\n\n      const orgError = console.error;\n      console.error = (...args) => {\n      if (args && args.length === 1 && isString(args[0]) && args[0].indexOf(\'You cannot change <Router routes>;\') > -1) {\n        // React route changed\n      } else {\n        // Log the error as normally\n        orgError.apply(console, args);\n      }\n      };\n    }\n\n    ' + componentsStr + '\n    const rootRoute = ' + rootRoute + '\n    module.exports = rootRoute';
  splitChildRoutes = '\n    import React from \'react\'\n    const rootRoute = ' + splitRootRoute + '\n    module.exports = rootRoute';
  _fs2.default.writeFileSync(directory + '/.intermediate-representation/child-routes.js', childRoutes);
  _fs2.default.writeFileSync(directory + '/.intermediate-representation/split-child-routes.js', splitChildRoutes);
};
var debouncedWriteChildRoutes = (0, _debounce3.default)(writeChildRoutes, 250);

var babelPlugin = function babelPlugin(_ref) {
  var t = _ref.types;

  return {
    visitor: {
      TemplateLiteral: function TemplateLiteral(path, state) {
        if (path.parentPath.parentPath.parentPath.type !== 'ExportNamedDeclaration') {
          return;
        }
        var exportPath = path.parentPath.parentPath.parentPath;
        var name = (0, _get3.default)(exportPath, 'node.declaration.declarations[0].id.name');
        if (name === 'pageQuery') {
          (function () {
            var quasis = (0, _get3.default)(path, 'node.quasis', []);
            var expressions = path.get('expressions');
            var chunks = [];
            quasis.forEach(function (quasi) {
              chunks.push(quasi.value.cooked);
              var expr = expressions.shift();
              if (expr) {
                chunks.push(expr.scope.bindings[expr.node.name].path.get('value').parentPath.node.init.quasis[0].value.cooked);
              }
            });
            var query = chunks.join('');
            console.time('graphql query time');
            var graphql = state.opts.graphql;

            //path.parentPath.replaceWithSourceString(`require('fixme.json')`);
          })();
        }
      }
    }
  };
};

// Queue for processing files
var q = (0, _queue2.default)(function (_ref2, callback) {
  var file = _ref2.file,
      graphql = _ref2.graphql,
      directory = _ref2.directory;

  var fileStr = _fs2.default.readFileSync(file, 'utf-8');
  var ast = void 0;
  try {
    ast = babylon.parse(fileStr, {
      sourceType: 'module',
      sourceFilename: true,
      plugins: ['asyncFunctions', 'jsx', 'flow', 'objectRestSpread']
    });
  } catch (e) {
    console.log('Failed to parse ' + file);
    console.log(e);
  }

  // Get query for this file.
  var query = void 0;
  (0, _babelTraverse2.default)(ast, {
    TemplateLiteral: function TemplateLiteral(path, state) {
      if (path.parentPath.parentPath.parentPath.type !== 'ExportNamedDeclaration') {
        return;
      }
      var exportPath = path.parentPath.parentPath.parentPath;
      var name = (0, _get3.default)(exportPath, 'node.declaration.declarations[0].id.name');
      if (name === 'pageQuery') {
        (function () {
          var quasis = (0, _get3.default)(path, 'node.quasis', []);
          var expressions = path.get('expressions');
          var chunks = [];
          quasis.forEach(function (quasi) {
            chunks.push(quasi.value.cooked);
            var expr = expressions.shift();
            if (expr) {
              chunks.push(expr.scope.bindings[expr.node.name].path.get('value').parentPath.node.init.quasis[0].value.cooked);
            }
          });
          query = chunks.join('');
        })();
      }
    }
  });
  var absFile = _path2.default.resolve(file);
  // Get paths for this file.
  var paths = [];
  (0, _globals.pagesDB)().forEach(function (value, key) {
    if (value.component === absFile) {
      paths.push(value);
    }
  });
  var pathsInfo = {
    componentPath: absFile,
    directory: directory,
    paths: paths,
    graphql: graphql
  };

  // Run queries for each page component.
  console.time('graphql query time');
  _promise2.default.all(paths.map(function (pathInfo) {
    var pathContext = void 0;
    // Mixin the path context to the top-level.
    // TODO do runtime check that user not passing in key that conflicts
    // w/ top-level keys e.g. path or component.
    if (pathInfo.context) {
      pathContext = (0, _extends3.default)({}, pathInfo, pathInfo.context);
    } else {
      pathContext = (0, _extends3.default)({}, pathInfo);
    }
    return graphql(query, pathContext).catch(function (error) {
      return console.log('graphql error', error);
    }).then(function (result) {
      if (result.errors) {
        console.log('graphql errors', result.errors);
      }
      // Combine the result with the path context.
      result.pathContext = pathInfo.context;
      var clonedResult = (0, _extends3.default)({}, result);
      result.pathContext = pathInfo;

      // Add result to page object.
      var page = (0, _globals.pagesDB)().get(pathInfo.path);
      var jsonName = (0, _kebabCase3.default)(pathInfo.path) + '.json';
      var internalComponentName = 'Component' + pascalCase(pathInfo.path);
      if (jsonName === '.json') {
        jsonName = 'index.json';
        internalComponentName = 'ComponentIndex';
      }
      page.jsonName = jsonName;
      page.internalComponentName = internalComponentName;
      (0, _globals.pagesDB)((0, _globals.pagesDB)().set(page.path, page));

      // Save result to file.
      var resultJSON = (0, _stringify2.default)(clonedResult, null, 4);
      _fs2.default.writeFileSync(directory + '/.intermediate-representation/json/' + jsonName, resultJSON);

      return null;
    });
  })).then(function () {
    console.log('rewrote JSON for queries for ' + absFile);
    console.timeEnd('graphql query time');
    // Write out new child-routes.js in the .intermediate-representation directory
    // in the root of your site.
    debouncedWriteChildRoutes();
    callback();
  });
}, 2);

module.exports = function (program, graphql, cb) {
  // Get unique array of component paths and then watch them.
  // When a component is updated, rerun queries.
  var components = (0, _uniq3.default)([].concat((0, _toConsumableArray3.default)((0, _globals.pagesDB)().values())).map(function (page) {
    return page.component;
  }));

  // If there's no components yet, call the callback early.
  if (components.length === 0) {
    cb();
  } else {
    q.drain = function () {
      // Only call callback once.
      q.drain = _noop3.default;
      cb();
    };
  }

  components.forEach(function (path) {
    q.push({ file: path, graphql: graphql, directory: program.directory });
  });

  // When not building, also start the watcher to detect when the components
  // change.
  if ((0, _last3.default)(program.parent.rawArgs) !== 'build') {
    var watcher = _chokidar2.default.watch(components, {
      //var watcher = chokidar.watch('/Users/kylemathews/programs/blog/pages/*.js', {
      ignored: /[\/\\]\./,
      persistent: true
    });

    watcher.on('add', function (path) {
      return q.push({ file: path, graphql: graphql, directory: program.directory });
    }).on('change', function (path) {
      return q.push({ file: path, graphql: graphql, directory: program.directory });
    }).on('unlink', function (path) {
      return q.push({ file: path, graphql: graphql, directory: program.directory });
    });
  }
};